import json
import pandas as pd
import datetime as dt
import requests as req

from . import dirs

from .cl_UsuarioCammesa import *

__all__ = ['Pronosticos']

class Pronosticos(UsuarioCammesa):
    
    def __check_son_numeros(self,iterable):
        es_numero = lambda x: isinstance(x,(int,float))
        return all(map(es_numero,iterable))
        
    def __check_son_string(self,iterable):
        es_string = lambda x: isinstance(x,str)
        return all(map(es_string,iterable))

    def __check_id_parques(self,id_parques):
        
        if isinstance(id_parques,(list,set,tuple)):
            todos_numeros = self.__check_son_numeros(id_parques)
            if not todos_numeros:
                raise TypeError('Todos los valores de ID_Parque deben ser del tipo Int o Float') 

        elif isinstance(id_parques,dict):
            todos_numeros = self.__check_son_numeros(id_parques.values())
            todos_strings = self.__check_son_string(id_parques.keys())

            if not todos_numeros:
                raise TypeError('Todos los valores del diccionario ID_Parque deben ser del tipo Int o Float')
            
            elif not todos_strings:
                raise TypeError('Todos las llaves del diccionario ID_Parque deben ser del tipo String') 
        
        elif isinstance(id_parques,(int,float,str)):
            pass  
        
        elif isinstance(id_parques,str):
            pass
        
        else:
            raise TypeError('La lista de IDs de parques para pedir pronósticos debe ser del tipo Tuple, List, Set o Dict.\nO bien ser un único número')

    def __hora_op(self,fecha):
        return fecha.hour +1

    def __fecha_op(self,fecha):  
        if (fecha.hour +1) == 24 :
            return (fecha.date() - dt.timedelta(days=1))
        else:
            return fecha.date()

    def _consultar_pronostico_viejo(self,id_parque,nombre_parque=None):
        '''
        Función para solicitar los pronósticos de los parques.
        La estructura de la respuesta es:
            respuesta = {ID_parque : respuesta_API}
                respuesta_API = {
                    ID_solicitud
                    ID_parque
                    pronóstico
                    }
            pronóstico = {time, kW}
        '''
        if not self.check_credenciales():
            raise SystemError('Imposible descargar pronósticos. Credenciales inválidas o sin conexión')

        url = f'https://cdspronosticos.cammesa.com/pronosticos/api/pronosticos/findPronostico?central={id_parque}'

        header = {'Content-Type': 'application/x-www-form-urlencoded',
                'Authorization': f'Bearer {self._token["access_token"]}'
                }
        
        r = req.get(url, headers=header)
        r_parsed = json.loads(r.text)
        
        #Retransmite errores de la API en caso de haber
        if 'pronosticos' not in r_parsed:
            raise SystemError(r_parsed['message'])

        #Continuar si todo está OK
        df = pd.DataFrame(data=r_parsed['pronosticos'])
        
        df['Park_ID'] = id_parque
        
        if isinstance(nombre_parque,str) or (nombre_parque is None):
            df['UC'] = nombre_parque
        else:
            raise TypeError('El parámetro "nombre_parque" debe ser del tipo String o None')
        
        return df
    
    def _consultar_pronostico_nuevo(self,id_parque,nombre_parque=None):
        if not self.check_credenciales():
            raise SystemError('Imposible descargar pronósticos. Credenciales inválidas o sin conexión')
    
        url = 'https://cdspronosticos.cammesa.com/v2/api/getPoints'
        header = {'Authorization': f'Bearer {self._token["access_token"]}',
                'Content-Type': 'application/json;charset=UTF-8'
            }
    
        #Esta nueva API tiene una TONELADA de datos que se están desaprovechando
        # ver cómo consumirlos en modo pronóstico y en RT
        
    def consultar_pronosticos(self,id_parques,pronostico='viejo'):

        self.__check_id_parques(id_parques)
        
        #Determina la función de consulta de pronóstico a utilizar
        if pronostico == 'viejo':
            funcion = self._consultar_pronostico_viejo 
        else:
            funcion = self._consultar_pronostico_nuevo
        
        #Listado de IDs, sin nombre
        if isinstance(id_parques,(list,tuple,set)):
            lista_dfs = [funcion(int(id)) for id in id_parques]
        
        #Listado de IDs, con nombre (diccionario)
        elif isinstance(id_parques,dict):
            lista_dfs = [funcion(int(id),n) for id, n in id_parques.items()]
            
        #Si pusieron un número en formato string y con cero: ejemplo 781.0
        else:
            id = int(float(id_parques))
            funcion(id)

        df = pd.concat(lista_dfs,ignore_index=True).rename(columns={'kw':'MWh'})
        
        df['time'] = pd.to_datetime(df['time'],format='%Y-%m-%dT%H:%M:%S')
        df['Fecha'] = df['time'].apply(self.__fecha_op)
        df['Hora'] = df['time'].apply(self.__hora_op)
        df['MWh'] = df['MWh'] /1000

        return df.loc[:,['Fecha','Hora','UC','MWh','Park_ID']]
    
    def descargar_pronosticos(self,id_parques,dir_descarga=None):
        if dir_descarga is None:
            dir_descarga = ''
        else:
            dir_descarga = dirs.check_dir(dir_descarga) + '\\'
        
        print(f'Se descargarán los pronósticos en:\n{dir_descarga}')
        
        fecha_salida = dt.datetime.now().strftime("%d-%m-%y %H-%M")
        nombre_archivo = f'Pronosticos_CAMMESA_{fecha_salida}.xlsx'
        ruta_salida = dir_descarga + nombre_archivo
        
        df = self.consultar_pronosticos(id_parques)
        df.to_excel(ruta_salida,index=False)


from . import dirs

from pathlib import Path
from cryptography.fernet import Fernet
from sqlalchemy import create_engine as ce

__all__ = ['SQLConnector',]

class SQLConnector():
    
    def __init__(
        self,
        host= None,
        port= None,
        usr = None,
        pwd = None,
        db = None,
        db_type = None,
        db_dialect = None,
        conectar=False
        ):
        
        self._host = None
        self._port = None
        self._usr = None
        self._pwd = None
        self._db = None
        self._db_type = None
        self._db_dialect = None
        self._conexion = None
        self._motor = None
        self._conectado = None

        has_some_init_params = host or port or usr or pwd or db or db_type
        if not has_some_init_params:
            try:
                self.__configurar_login()
            except:
                print('No se logró leer automáticamente la configuración privada del conector SQLL')
            
        has_all_init_params = host and port and usr and pwd and db and db_type
        if has_all_init_params:
            self.host = host 
            self.port = port 
            self.usr = usr 
            self.pwd = pwd 
            self.db = db 
            self.db_type = db_type
            self.db_type = db_dialect #Opcional
            
        if has_some_init_params and not has_all_init_params:
            print('Configuración del conector SQL incompleta, por favor ingresar los parámetros:')
            print('host')
            print('port')
            print('user')
            print('password')
            print('db')
            print('db_type')
            print('db_dialect')
        
        all_params_configured = self.host and self.port and self.usr and self.pwd and self.db and self.db_type
        if all_params_configured:
            self.motor = self.crear_motor()
        
        self.conectado = False
        if conectar:
            self.conectar()
        
    @property
    def host(self):
        return self._host
    
    @host.setter
    def host(self,val):
        self._host = val

    @property
    def port(self):
        return self._port
    
    @port.setter
    def port(self,val):
        self._port = val

    @property
    def usr(self):
        return self._usr
    
    @usr.setter
    def usr(self,val):
        self._usr = val
    
    @property
    def pwd(self):
        return self._pwd
    
    @pwd.setter
    def pwd(self,val):
        self._pwd = val
    
    @property
    def db(self):
        return self._db
    
    @db.setter
    def db(self,val):
        self._db = val

    @property
    def db_type(self):
        return self._db_type

    @db_type.setter
    def db_type(self,val):
        self._db_type = val
    
    @property
    def db_dialect(self):
        return self._db_dialect
    
    @db_dialect.setter
    def db_dialect(self,val):
        self._db_dialect = val
    
    @property
    def motor(self):
        return self._motor
    
    @motor.setter
    def motor(self,val):
        self._motor = val
    
    @property
    def conexion(self):
        return self._conexion
    
    @conexion.setter
    def conexion(self,val):
        self._conexion = val
        if not val is None:
            self.conectado = 'actualizar_status'
    
    @property
    def conectado(self):
        return self._conectado
    
    @conectado.setter
    def conectado(self,val):
        '''Note that "val" is not used'''
        try:
            self._conectado = not self.conexion.closed
        except:
            self._conectado = val
    
    def __leer_parametros_encriptados(self):
        '''Lee los parámetros encriptados para conectarse a la BD.'''

        ruta = dirs.get_dc_cfg() 
        file_cfg = Path(ruta + '\\' + 'cfg.db')
        
        txt_enc = file_cfg.read_text()
        
        key = 'chcoJbbFgDtz1dZIDg9g2JIfcCcTlG33q8I3SIJEixE='
        f = Fernet(key)

        txt = f.decrypt(txt_enc)
        txt = txt.decode()
        
        return txt

    def __obtener_parametros(self,):
        '''Convierte el texto de parámetros a un diccionario'''
        
        params = self.__leer_parametros_encriptados()
        
        d = locals()
        exec(f"dct = {params}",globals(), d)
        return d['dct']

    def __configurar_login(self):

        p = self.__obtener_parametros()

        self.host = p['host']
        self.port = p['port']
        self.usr = p['usr']
        self.pwd = p['pwd']
        self.db = p['db']
        self.db_type = p['db_type']
        self.db_dialect = p['db_dialect']

    def crear_motor(
        self,
        host = None, 
        port = None,
        usr = None, 
        pwd = None,
        db = None,
        db_type = None,
        db_dialect = None,
        echo=False,
        future=False
    ):
        '''Crea un objeto "Engine" para interactuar con la base de datos.'''
        if host is None:
            host = self.host
            
        if port is None:
            port = self.port
              
        if usr is None:
            usr = self.usr
            
        if pwd is None:
            pwd = self.pwd
            
        if db is None:
            db = self.db
            
        if db_type is None:
            db_type = self.db_type
            
        if db_dialect is None:
            db_dialect = self.db_dialect
            
        if db_dialect:
            url = f"{db_type}+{db_dialect}://{usr}:{pwd}@{host}:{port}/{db}"
        else:
            url = f"{db_type}://{usr}:{pwd}@{host}:{port}/{db}"

        return ce(url,echo=echo,future=future)

    def conectar(self,mensajes=False):
        '''Intenta conectarse a la base de datos'''
        self.conexion = self.motor.connect()
        
        if mensajes:
            if self.conectado:
                print('Conexión exitosa')
            else:
                print("Imposible conectarse a la Base de Datos del CROM")

    def desconectar(self,mensajes=False):
        
        if self.conectado:
            self._conexion.close()
            self.conexion = self._conexion
        
        if mensajes:
            if not self.conectado :
                print('Desconexión exitosa')
            else:
                print("Error al desconectar.")

    def checkear_conexion(self,mensajes=False):
        if not self.conectado:
            try:
                self.conectar(mensajes=mensajes)
            except:
                pass

        return self.conectado


import json
import requests as req

__all__ = ['UsuarioCammesa']

class UsuarioCammesa():
    def __init__(self,usr=None,pwd=None):
        # Se puede dejar hardcodeado o solicitar usr y pwd
        self._usr = usr
        self._pwd = pwd
        self._header = {'Content-Type': 'application/x-www-form-urlencoded'}
        self._scrt = "6a5fd084-fc67-4468-829c-0ff3a68f6ca3"   #Código adicional, provisto por CAMMESA para cada API
        self._token = None

    @property
    def usr(self):
        return self._usr

    @property
    def pwd(self):
        return self._pwd
    
    @usr.setter
    def usr(self,val):
        if not isinstance(val,str):
            raise TypeError('El nombre de usuario debe ser del tipo String')

    @pwd.setter
    def pwd(self,val):
        if not isinstance(val,str):
            raise TypeError('La contraseña debe ser del tipo String')

    def log_in(self):
        url = "https://keycloak.cammesa.com/auth/realms/Cammesa/protocol/openid-connect/token"
        params = {
            'username': self.usr,
            'password': self.pwd,
            'grant_type': 'password',
            'client_id': 'pronosticos',
            'client_secret': self._scrt
        }
        
        #Solicitud de credenciales para la API
        r = req.post(url,headers = self._header,data=params)
        r_parsed = json.loads(r.text)
        
        #Chequear error de usuario y contraseña en la respuesta de la API
        if 'error' in r_parsed:
            print('No se pudo iniciar sesión: Credenciales inválidas')
            self._token = None
        elif not ('access_token' in r_parsed):
            print('No se pudo iniciar sesión: Error desconocido. ')
            self._token = None
        else:
            self._token = r_parsed

    def log_out(self):

        url = 'https://keycloak.cammesa.com/auth/realms/Cammesa/protocol/openid-connect/logout'
        params = {
            'd_refresh_token':self._token['refresh_token'],
            'client_id': 'pronosticos',
            'client_secret': self._scrt}

        req.post(url,headers = self._header,data=params)

    def __check_usr_pwd(self):
        if self.usr is None :
            raise ValueError('Debe ingresar usuario vía la funcion .usr')
        elif self.pwd is None:
            raise ValueError('Debe ingresar contraseña vía la funcion .pwd')
        else:
            return True

    def check_credenciales(self):
        if self.__check_usr_pwd():
            if self._token is None:
                print(f'Forzando inicio de sesión para el usuario {str(self.usr)}')
                self.log_in()
                if self._token is None:
                    print('No se pudo forzar inicio de sesión. Abortando proceso.')
                    return False
        return True


from pathlib import Path


__all__ = ['__CarpetaServicios']


mensaje = """No se ha encontrado la carpeta hacia la nube de BLC.\n
Tendrá que indicar la ruta hacia la carpeta de Servicios manualmente.
Para ello por favor ejecutar el comando: blctools.indicar_ruta('ruta completa')
Mientras tanto se trabajará en la ruta del script actual"""


class __CarpetaServicios():
       
    def __init__(self):
        self._ruta = str(Path.cwd())
        self.ruta_encontrada = False
        
        self.__buscar_dir_segun_carpeta_usuario()
        
        if not self.ruta_encontrada:
            self.__buscar_dir_segun_archivo_py()
            
        if not self.ruta_encontrada:
            self.print_warning
        
    def __buscar_dir_segun_archivo_py(self):
        
        dir_raiz_split = self.ruta.split('\\')
        dir_rel_1 = r'OneDrive - BLCGES\Documentos Produccion\Servicios'
        dir_rel_2 = r'OneDrive - BLCGES\Servicios'
        
        for dir_rel in [dir_rel_1, dir_rel_2]:
            if dir_rel in self.ruta:
                index_dir_servicios = dir_raiz_split.index('Servicios')
                seleccion = dir_raiz_split[0:index_dir_servicios+1]
                
                dir_servicios = '\\'.join(seleccion)
                
                self.ruta = dir_servicios
                self.ruta_encontrada = True
                break
        
    def __buscar_dir_segun_carpeta_usuario(self):
        dir_usr = str(Path.home())
        dir_rel_1 = r'OneDrive - BLCGES\Documentos Produccion\Servicios'
        dir_rel_2 = r'OneDrive - BLCGES\Servicios'
        
        for dir_rel in [dir_rel_1,dir_rel_2]:
            dir_full = dir_usr + '\\' + dir_rel
            path_full = Path(dir_full)
            
            if path_full.is_dir() and path_full.exists():
                self.ruta = dir_full
                self.ruta_encontrada = True
                break

    def print_warning(self):
        if __name__ != '__main__':
            print(mensaje)
            
    @property
    def ruta(self):
        return self._ruta
    
    @ruta.setter
    def ruta(self,val):
        
        if isinstance(val,Path):
            val = str(Path(val))
            
        elif not isinstance(val,str):
            raise TypeError('La ruta ingresada debe ser del tipo String o pathlib.Path')
        
        else:
            obj_path = Path(val)
            
            if not obj_path.is_dir():
                raise ValueError('La ruta especificada aparenta no ser un directorio')
            elif not obj_path.exists():
                raise ValueError('La ruta especificada no existe')
            elif not obj_path.is_absolute():
                raise ValueError('La ruta especificada es relativa, se requiere una ruta completa')
        
            self._ruta = val


import json
import pandas as pd
import datetime as dt
import requests as req

from . import dirs
from . import fechas

__all__ = ['ApiCammesa',]

class ApiCammesa():
    '''Esta clase contiene principalmente métodos y pocos atributos.
    Sus atributos guardan consultas realizadas a la API de CAMMESA en la memoria.'''
    
    def __init__(self):
        self._consulta_total = None
        self._consulta_iniciales = None
        self._consulta_ultimos = None
        self._consulta_custom = None
        self._filtro = None
        self._filtro_val_pos_default = ['iniciales','ultimos',None]
        self._filtro_custom_dict = {}
        self._filtro_func_custom = None
        self._rts_vigentes = None
        
    @property
    def reportes_vigentes(self):
        if self._rts_vigentes is None:
            self._rts_vigentes = self.__cargar_rpts_vigentes()

        return self._rts_vigentes

    @property
    def filtro(self):
        return self._filtro
    
    @filtro.setter
    def filtro(self,val):
        self._filtro = self.check_filtro(val)
    
    @property
    def consulta(self):

        if  self.filtro == 'iniciales':
            return self._consulta_iniciales
        elif self.filtro == 'ultimos':
            return self._consulta_ultimos
        elif self.filtro is None:
            return self._consulta_total
        else:
            return self._consulta_custom

    @property
    def filtro_custom_dict(self):
        return self._filtro_custom_dict

    @filtro_custom_dict.setter
    def filtro_custom_dict(self,val):
        if isinstance(val,dict):
            #Los filtros son case-sensitive y deben estar todos en minuscula
            val = self.__llaves_dict_minuscula(val)
            if self.__check_funciones_dict(val):
                self._filtro_custom_dict = val
            else:
                raise TypeError('Los valores del diccionario de filtros custom deben ser funciones')
        else:
            raise TypeError('El filtro custom debe ser un diccionario')
        
    def __llaves_dict_minuscula(self,dic):
        
        old_k = dic.keys()
        
        try:
            new_k = map(lambda x: x.lower(),old_k)
            
        except:
            raise TypeError('Todas las llaves del diccionario de filtros custom deben ser del tipo String')
        
        mapeo_k = dict(zip(old_k,new_k))
        
        return {mapeo_k[k]:v for k,v in dic.items()}
    
    def __check_funciones_dict(self,dic):
        return all(map(callable,dic.values()))
    
    def check_filtro(self,val):
        
            if isinstance(val,dict):
                self.filtro_custom_dict = val

            #Armar una lista de todos los valores posibles
            valores_posibles_totales = self._filtro_val_pos_default + list(self._filtro_custom_dict.keys())

            if isinstance(val,str):
                val = val.lower()
                if not val in valores_posibles_totales:
                    print(f'Se ingresó un parámetro "Filtro" igual a: {val}')
                    raise ValueError(f'El valor de "filtro" debe estar entre {valores_posibles_totales}')

                elif val in self._filtro_custom_dict:
                    func = self.filtro_custom_dict[val]
                    if not callable(func):
                        raise TypeError('La función personalizada ingresada no es ejecutable. Ejemplo: func()')
                    else:
                        self._filtro_func_custom = func
                    
            elif val is None:
                pass
            
            else:
                print(f'Filtro vale actualmente: {val}, no es un string, es: {type(val)}')
                raise TypeError('La variable filtro debe ser None, String o una función personalizada '\
                    'en caso de ser un string distinto a "iniciales" o "ultimos"')

            return val
    
    def __cargar_rpts_vigentes(self):
        
        url ='https://api.cammesa.com/pub-svc/public/catalogoPublicacionesVigentes'

        r = req.get(url)
        r_parsed = json.loads(r.text)

        return pd.DataFrame(list(r_parsed))
        
    def __get_fechas_consulta(self,fecha_i, fecha_f=None):
        
        fi, ff = fechas.validar_fechas(fecha_i, fecha_f)
        fi = fi.strftime('%Y-%m-%d')
        ff = ff.strftime('%Y-%m-%d')
        
        return fi, ff
    
    def __docs_by_nemo_rango(self,fecha_i, fecha_f=None,nemo_rpt=''):
        '''Consulta de reportes a la API de CAMMESA'''
        
        fi, ff = self.__get_fechas_consulta(fecha_i, fecha_f)
        
        url = 'https://api.cammesa.com/pub-svc/public/' + \
            'findDocumentosByNemoRango?' + \
            f'fechadesde={fi}T03%3A00%3A00.000%2B00%3A00&' + \
            f'fechahasta={ff}T03%3A00%3A00.000%2B00%3A00&' + \
            f'nemo={nemo_rpt}'
        
        r = req.get(url)
        
        return json.loads(r.text)
    
    def __get_link_descarga(self,archivo,id_doc,nemo_rpt):
        '''Simplemente devuelve un string que sirve para descargar un archivo'''
        
        if not(isinstance(archivo,str) and isinstance(id_doc,str) and isinstance(nemo_rpt,str)):
            raise ValueError('Las variables de entrada deben ser todas del tipo string')
        
        link_descarga = 'https://api.cammesa.com/pub-svc/public/' + \
                        'findAttachmentByNemoId?' + \
                        f'attachmentId={archivo}&' + \
                        f'docId={id_doc}&' + \
                        f'nemo={nemo_rpt}'
        
        return link_descarga

    def consultar(self,fecha_i, fecha_f, nemo_rpt,exportar_consulta=False,dir_consulta=None,filtro=None):
        '''Se ingresa con objetos datetime de fechas.
        Devuelve un dataframe de pandas con todos los archivos encontrados en dicho rango.
        NO FILTRA ni elimina duplicados en caso de existir varias versiones del mismo reporte (final/inicial)'''

        if filtro == False:
                print('Sin parámetro ni atributo detectado de "filtro". Colocando filtro = ultimos.')
                self.filtro = 'ultimos'
        else:
            self.filtro = filtro

        print("Procediendo a consultar API de CAMMESA.")
        r_parsed = self.__docs_by_nemo_rango(fecha_i, fecha_f,nemo_rpt=nemo_rpt)

        df_inicial = pd.DataFrame(list(r_parsed))
        df_inicial['adjuntos'] = df_inicial['adjuntos'].apply(list)
        df_inicial['fecha'] = pd.to_datetime(df_inicial['fecha'], format='%d/%m/%Y')
        df_inicial['version'] = pd.to_datetime(df_inicial['version'],format='%Y-%m-%dT%H:%M:%S.000-03:00')
        
        registros_adjuntos = [x[0] for x in df_inicial['adjuntos']]
        df_adjuntos = pd.DataFrame(registros_adjuntos).rename(columns={'id':'Archivo'})
        df_adjuntos['Archivo'] = df_adjuntos['Archivo'].str.upper()
        
        df_total  = df_inicial\
                    .drop(labels='adjuntos',axis=1)\
                    .join(df_adjuntos)\
                    .sort_values(by=['Archivo','version'],ascending=[1,1])
        
        self._consulta_total = df_total
        self._consulta_iniciales = df_total.drop_duplicates(subset='Archivo',keep='first')
        self._consulta_ultimos = df_total.drop_duplicates(subset='Archivo',keep='last')

        if isinstance(self.filtro,str) :
            if self.filtro in self.filtro_custom_dict:
                #Esto implica que el filtro solicitado es una función custom 
                self._consulta_custom = self._filtro_func_custom(df_total)
            
        if exportar_consulta:
            print("Exportando consulta realizada a la API de CAMMESA.")
            
            fecha_consulta = dt.datetime.now().strftime('%Y.%m.%d %H.%M.%S')
            nombre_archivo = f"Consulta {nemo_rpt} {fecha_consulta}.xlsx"
            
            if dir_consulta is None:
                dir_consulta = dirs.raiz
            else:
                dir_consulta = dirs.check_dir(dir_consulta)
                
            print(f"Ruta de exportación: {dir_consulta}")
            self.consulta.to_excel(dir_consulta + '\\' + nombre_archivo)
        
        del df_inicial, df_adjuntos
        
        return self.consulta
        
    def __descargar_reporte(self,archivo,id_doc,nemo_rpt,dir_descarga):
        '''Descarga un único archivo de la API de CAMMESA.'''
        
        if dir_descarga == None:
            dir_descarga = dirs.raiz + '\\00 ZIP'
        else:
            dir_descarga = dirs.check_dir(dir_descarga)
        
        link_descarga = self.__get_link_descarga(archivo,id_doc,nemo_rpt)
        ruta_descarga = dir_descarga + "\\" + archivo

        print(f"Descargando: {archivo} {nemo_rpt}.")
        respuesta = req.get(link_descarga)

        with open(ruta_descarga, 'wb') as f:
            f.write(respuesta.content)

    def descargar_consulta(self,dir_descarga):
        '''Toma un dataframe de pandas formateado según la función consultar().
        Recorre el mismo y descarga todos los archivos en la carpeta designada como "dir_descarga"'''

        print("Procediendo a descargar archivos.")
        print(f"Ruta de descarga:{dir_descarga}")
        
        df = self.consulta
        registro_de_archivos = df.loc[:,['Archivo','id','nemo']].to_dict(orient='records')

        for r in registro_de_archivos:
            self.__descargar_reporte(
                r['Archivo'],
                r['id'],
                r['nemo'],
                dir_descarga
                )

    def descargar(self,
                  fecha_i, fecha_f,nemo_rpt, 
                  dir_descarga=None, 
                  exportar_consulta=False,dir_consulta=None, 
                  filtro=False
                  ):
        """Toma un dataframe de pandas formateado según la función consultar() del módulo cammesa_api.py.
        Recorre el mismo y descarga todos los archivos en la carpeta designada como 'dir_descarga' """
                   
        self.consultar(
            fecha_i, fecha_f, nemo_rpt,
            exportar_consulta=exportar_consulta, 
            dir_consulta=dir_consulta,
            filtro=filtro
            )
        
        
        self.descargar_consulta(dir_descarga)